<?php
/*
  $Id$

  QTPro 7.3.0 Phoenix
  by @raiwa
  info@oscaddons.com
  www.oscaddons.com

  Copyright (c)2021 Rainer Schmied
  Based on prior works released under the GNU General Public License

  osCommerce, Open Source E-Commerce Solutions
  http://www.oscommerce.com

  Copyright (c) 2020 osCommerce

  Released under the GNU General Public License
*/

  class pi_qtpro_options extends abstract_module {

    const CONFIG_KEY_BASE = 'PI_QTPRO_OPTIONS_';

    public $group = 'pi_modules_c';
    public $content_width;

    function __construct() {
      parent::__construct();

      $this->group = basename(dirname(__FILE__));

      $this->description .= '<div class="alert alert-warning">' . MODULE_CONTENT_BOOTSTRAP_ROW_DESCRIPTION . '</div>';
      $this->description .= '<div class="alert alert-info">' . cm_pi_modular::display_layout() . '</div>';

      $this->description .= '<div class="secWarning"><a target="_blank" href="https://www.oscaddons.com/en/commercial-add-ons/qtpro-update-product-price-p-56.html"><u>GET QTpro Update Product Price</u></a><br>
                            - Dynamically update the shown Product Price depending on the selected options.<br>
                              <a target="_blank" href="https://www.oscaddons.com"><u>www.oscaddons.com</u></a>
                            </div>';

      if (!file_exists(DIR_FS_CATALOG . 'templates/' . TEMPLATE_SELECTION . '/includes/hooks/shop/siteWide/qtPro.php')) {
        $this->description = '<div class="secWarning">' . PI_QTPRO_OPTIONS_ERROR_HOOK . '</div>' . $this->description;
        $this->enabled = false;
      }

      if ( $this->enabled ) {
        $this->group = 'pi_modules_' . strtolower(PI_QTPRO_OPTIONS_GROUP);
        $this->content_width = (int)PI_QTPRO_OPTIONS_CONTENT_WIDTH;
      }

    }

    function getOutput() {
      global $product_info;

      $content_width = (int)PI_QTPRO_OPTIONS_CONTENT_WIDTH;

      $product_exists = false;
      if (method_exists('product_info','product_exists') && isset($product_info) && $product_info->product_exists() === true && $product_info->has_options() === true ) {
        $product_exists = true;
      } else {
        $products_attributes_query = tep_db_query(sprintf(<<<'EOSQL'
SELECT count(*) AS total
  FROM products_options popt, products_attributes patrib
  WHERE patrib.products_id = %s
    AND patrib.options_id = popt.products_options_id
    AND popt.language_id = %s
EOSQL
        , (int)$_GET['products_id'], (int)$_SESSION['languages_id']));

        $products_attributes = $products_attributes_query->fetch_assoc();
        if ($products_attributes['total'] > 0)
          $product_exists = true;
      }

      if ($product_exists === true) {
        $products_id=(preg_match("/^\d{1,10}(\{\d{1,10}\}\d{1,10})*$/", $_GET['products_id']) ? $_GET['products_id'] : (int)$_GET['products_id']);
        require('includes/system/override/qtpro/pad_' . PI_QTPRO_OPTIONS_PLUGIN . '.php');
        $class = 'pad_' . PI_QTPRO_OPTIONS_PLUGIN;
        $pad = new $class($products_id);
        $qtpro_options = $pad->draw();

        $tpl_data = ['group' => $this->group, 'file' => __FILE__];
        include 'includes/modules/block_template.php';
      }
    }

    protected function get_parameters() {
      return [
        'PI_QTPRO_OPTIONS_VERSION_INSTALLED' => [
          'title' => 'Current Version',
          'value' => '7.3.0. Phoenix',
          'desc' => 'Version info. It is read only',
          'set_func' => 'cm_pi_qtpro_options::readonly(',
        ],
        'PI_QTPRO_OPTIONS_STATUS' => [
          'title' => 'Enable QTPRO Options Module',
          'value' => 'True',
          'desc' => 'Should this module be shown on the product info page?',
          'set_func' => "tep_cfg_select_option(['True', 'False'], ",
        ],
        'PI_QTPRO_OPTIONS_GROUP' => [
          'title' => 'Module Display',
          'value' => 'C',
          'desc' => 'Where should this module display on the product info page?',
          'set_func' => "tep_cfg_select_option(['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I'], ",
        ],
        'PI_QTPRO_OPTIONS_CONTENT_WIDTH' => [
          'title' => 'Content Width',
          'value' => '12',
          'desc' => 'What width container should the content be shown in?',
          'set_func' => "tep_cfg_select_option(['12', '11', '10', '9', '8', '7', '6', '5', '4', '3', '2', '1'], ",
        ],
        'PI_QTPRO_OPTIONS_REMOVE_DATA' => [
          'title' => 'Uninstall Removes Database entries',
          'value' => 'False',
          'desc' => 'Do you want to remove the QT Pro database entries when uninstall the module?<br>All attribute quantities and other QT Pro database entries will be lost. Use only if you just installed QT Pro for testing purpose and you are sure not to use QT Pro any more.',
          'set_func' => "tep_cfg_select_option(['True', 'False'], ",
        ],
        'PI_QTPRO_OPTIONS_PLUGIN' => [
          'title' => 'Product Info Attribute Display Plugin',
          'value' => 'multiple_dropdowns',
          'desc' => 'The plugin used for displaying attributes on the product information page.',
          'set_func' => "tep_cfg_select_option(['multiple_dropdowns', 'sequenced_dropdowns', 'single_dropdown', 'single_radioset'], ",
        ],
        'PI_QTPRO_OPTIONS_SHOW_OUT_OF_STOCK' => [
          'title' => 'Show Out of Stock Attributes',
          'value' => 'True',
          'desc' => 'If True:</strong> Attributes that are out of stock will be displayed.<br>If False: Attributes that are out of stock will <em>not</em> be displayed.<br>Has no effect for sequenced dropdowns.<br>Default is True.',
          'set_func' => "tep_cfg_select_option(['True', 'False'], ",
        ],
        'PI_QTPRO_OPTIONS_MARK_OUT_OF_STOCK' => [
          'title' => 'Mark Out of Stock Attributes',
          'value' => 'Right',
          'desc' => 'Controls how out of stock attributes are marked as out of stock.',
          'set_func' => "tep_cfg_select_option(['None', 'Right', 'Left'], ",
        ],
        'PI_QTPRO_OPTIONS_OUT_OF_STOCK_MSGLINE' => [
          'title' => 'Display Out of Stock Message Line',
          'value' => 'True',
          'desc' => 'If True: If an out of stock attribute combination is selected by the customer, a message line informing on this will displayed.<br>Used for multiple dropdowns only.',
          'set_func' => "tep_cfg_select_option(['True', 'False'], ",
        ],
        'PI_QTPRO_OPTIONS_NO_ADD_OUT_OF_STOCK' => [
          'title' => 'Prevent Adding Out of Stock to Cart',
          'value' => 'True',
          'desc' => 'If True: Customer will not be able to ad a product with an out of stock attribute combination to the cart. A javascript form will be displayed.',
          'set_func' => "tep_cfg_select_option(['True', 'False'], ",
        ],
        'PI_QTPRO_OPTIONS_COMBINE_NON_STOCKED' => [
          'title' => 'Combine untracked options',
          'value' => 'True',
          'desc' => 'For single drop down or singel radio: combine also options which stock is not tracked.',
          'set_func' => "tep_cfg_select_option(['True', 'False'], ",
        ],
        'PI_QTPRO_OPTIONS_ACTUAL_PRICE_PULL_DOWN' => [
          'title' => 'Use Actual Price Pull Downs',
          'value' => 'False',
          'desc' => 'NOTE: If you have more than one option per product, this can only be used with a satisfying result with single_dropdown or single_radioset.',
          'set_func' => "tep_cfg_select_option(['True', 'False'], ",
        ],
        'PI_QTPRO_OPTIONS_SHOW_PLEASE_SELECT' => [
          'title' => 'Show default \'Please Select\' option',
          'value' => 'True',
          'desc' => 'If True: \'Please Select\' will be shown by default option. (multiple and single drop-down only)<br> If no other option is selected, click on \'Add to Cart\' button will show an alert.',
          'set_func' => "tep_cfg_select_option(['True', 'False'], ",
        ],
        'PI_QTPRO_OPTIONS_SORT_ORDER' => [
          'title' => 'Sort Order',
          'value' => '80',
          'desc' => 'Sort order of display. Lowest is displayed first.',
        ],
      ];
    }

    function install($parameter_key = null) {
      parent::install($parameter_key);

      // BEGIN options images support
      if ( defined('MODULE_HEADER_TAGS_BOOTSTRAP_SELECT_STATUS') && MODULE_HEADER_TAGS_BOOTSTRAP_SELECT_STATUS == 'True' ) {
        // ADD FIELD for options images
        if (tep_db_num_rows(tep_db_query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA='". DB_DATABASE . "' AND TABLE_NAME='products_attributes' AND COLUMN_NAME LIKE 'option_image'")) != 1 ) {
          tep_db_query("AlTER TABLE products_attributes ADD option_image varchar(127) NULL default NULL");
        }
        // ADD FIELD for options sort order
        if (tep_db_num_rows(tep_db_query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA='". DB_DATABASE . "' AND TABLE_NAME='products_attributes' AND COLUMN_NAME LIKE 'sort_order'")) != 1 ) {
        tep_db_query("ALTER TABLE products_attributes ADD sort_order int(11) NOT NULL default 0");
        }
        // ADD FIELD for options values images
        if (tep_db_num_rows(tep_db_query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA='". DB_DATABASE . "' AND TABLE_NAME='products_options_values' AND COLUMN_NAME LIKE 'products_options_values_image'")) != 1 ) {
          tep_db_query("AlTER TABLE products_options_values ADD products_options_values_image varchar(127) NULL default NULL");
        }
        // Create image directory for options images
        if(!is_dir(DIR_FS_CATALOG_IMAGES . 'options/'))mkdir(DIR_FS_CATALOG_IMAGES . 'options/', 0755);
      }
      // END options images support

      // Add new column to products_options to indicate if stock should be tracked for an option
      if (tep_db_num_rows(tep_db_query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA='". DB_DATABASE . "' AND TABLE_NAME='products_options' AND COLUMN_NAME LIKE 'products_options_track_stock'")) != 1 ) {
        tep_db_query("ALTER TABLE `products_options` ADD COLUMN `products_options_track_stock` TINYINT(4) DEFAULT '0' NOT NULL AFTER `products_options_name`");
      }
      // Add new column to orders_products to track attributes to make it possible to delete an order and restock
      if (tep_db_num_rows(tep_db_query("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA='". DB_DATABASE . "' AND TABLE_NAME='orders_products' AND COLUMN_NAME LIKE 'products_stock_attributes'")) != 1 ) {
        tep_db_query("ALTER TABLE `orders_products` ADD COLUMN `products_stock_attributes` VARCHAR(255) DEFAULT NULL AFTER `products_quantity`");
      }
      // Create new table to track stock for products attributes
      tep_db_query( "CREATE TABLE if not exists `products_stock` (`products_stock_id` INT(11) NOT NULL AUTO_INCREMENT,
                                                                  `products_id` INT(11) DEFAULT '0' NOT NULL ,
                                                                  `products_stock_attributes` VARCHAR(255) NOT NULL,
                                                                  `products_stock_quantity` INT(11) DEFAULT '0' NOT NULL ,
                                                                  PRIMARY KEY (products_stock_id),
                                                                  UNIQUE idx_products_stock_attributes (`products_id`,`products_stock_attributes`))");
      // Delete old configuration entries
      tep_db_query("DELETE FROM configuration_group WHERE configuration_group_title='Prod Info (QTPro)'");
      tep_db_query("DELETE FROM configuration WHERE configuration_key IN ('" . implode("', '", $this->old_keys()) . "')");
    }

    function remove($parameter_key = null) {
      parent::remove($parameter_key);
      if ( defined('PI_QTPRO_OPTIONS_REMOVE_DATA') && PI_QTPRO_OPTIONS_REMOVE_DATA == 'True' ) {
      	tep_db_query("ALTER TABLE `products_options` DROP `products_options_track_stock`");
      	tep_db_query("ALTER TABLE `orders_products` DROP `products_stock_attributes`");
      	// BEGIN options images support
        tep_db_query("ALTER TABLE products_options_values DROP COLUMN IF EXISTS products_options_values_image");
        tep_db_query("ALTER TABLE `products_attributes` DROP COLUMN IF EXISTS `option_image`");
        // END options images support
      	tep_db_query("DROP TABLE IF EXISTS `products_stock`");
      }
    }

    function old_keys() {
      $KeysArray =  array('PRODINFO_ATTRIBUTE_PLUGIN',
                          'PRODINFO_ATTRIBUTE_SHOW_OUT_OF_STOCK',
                          'PRODINFO_ATTRIBUTE_MARK_OUT_OF_STOCK',
                          'PRODINFO_ATTRIBUTE_OUT_OF_STOCK_MSGLINE',
                          'PRODINFO_ATTRIBUTE_NO_ADD_OUT_OF_STOCK',
                          'PRODINFO_ATTRIBUTE_ACTUAL_PRICE_PULL_DOWN',
                          'PRODINFO_ATTRIBUTE_DISPLAY_STOCK_LIST');
      return $KeysArray;
    }

    public static function readonly($value) {
      return $value;
    }

  }

