<?php
/*
  $Id$

  2020 QTPro 7.3.0 CE Phoenix
  by @raiwa
  info@oscaddons.com
  www.oscaddons.com

  Copyright (c)2021 Rainer Schmied
  Based on prior works released under the GNU General Public License

  osCommerce, Open Source E-Commerce Solutions
  http://www.oscommerce.com

  Copyright (c) 2020 osCommerce

  Released under the GNU General Public License
*/

  class pi_qtpro_table extends abstract_module {

    const CONFIG_KEY_BASE = 'PI_QTPRO_TABLE_';

    public $group = 'pi_modules_c';
    public $content_width;

    function __construct() {
      parent::__construct();

      $this->group = basename(dirname(__FILE__));

      if (!defined('MODULE_CONTENT_PI_QTPRO_OPTIONS_STATUS') && !defined('PI_QTPRO_OPTIONS_STATUS')) {
        $this->description .=   '<div class="secWarning">' . PI_QTPRO_TABLE_WARNING . '<br>
                                <a href="modules_content.php?module=cm_pi_qtpro_options&action=install">' . PI_QTPRO_TABLE_INSTALL_NOW . '</a></div>';
      }

      if (!file_exists(DIR_FS_CATALOG . 'templates/' . TEMPLATE_SELECTION . '/includes/hooks/shop/siteWide/qtPro.php')) {
        $this->description = '<div class="secWarning">' . PI_QTPRO_TABLE_ERROR_HOOK . '</div>' . $this->description;
        $this->enabled = false;
      }

      $this->description .= '<div class="secWarning">' . MODULE_CONTENT_BOOTSTRAP_ROW_DESCRIPTION . '</div>';

      if ( $this->enabled ) {
        $this->group = 'pi_modules_' . strtolower(PI_QTPRO_TABLE_GROUP);
        $this->content_width = (int)PI_QTPRO_TABLE_CONTENT_WIDTH;
      }

    }

    function getOutput() {
      global $product_info;

      $content_width = (int)PI_QTPRO_TABLE_CONTENT_WIDTH;

      /////   MISSION CODENAME: "GET INFORMATION" STARTS HERE   /////
      //Get the products_price and products_tax_class_id
      $products_facts_query = tep_db_query(sprintf(<<<'EOSQL'
SELECT IF(s.status, s.specials_new_products_price, p.products_price) AS products_price, p.products_tax_class_id
  FROM products p
    LEFT JOIN specials s ON p.products_id = s.products_id
  WHERE p.products_id = %s
EOSQL
        , (int)$_GET['products_id']));

      $products_facts = tep_db_fetch_array($products_facts_query);

      // Get the stocklevels
      $products_stock_query = tep_db_query(sprintf(<<<'EOSQL'
SELECT products_stock_attributes, products_stock_quantity
  FROM products_stock
  WHERE products_id = %s
  ORDER BY products_stock_attributes
EOSQL
        , (int)$_GET['products_id']));

      // get the option names
      $products_options_name_query = tep_db_query(sprintf(<<<'EOSQL'
SELECT distinct popt.products_options_id, popt.products_options_name
  FROM products_options popt, products_attributes patrib
  WHERE patrib.products_id = %s
    AND patrib.options_id = popt.products_options_id
    AND popt.products_options_track_stock = '1'
    AND popt.language_id = %s
  ORDER BY popt.products_options_id
EOSQL
        , (int)$_GET['products_id'],(int)$GLOBALS['languages_id']));

      // build array of attributes price delta
      $attributes_price = array();
      $products_attributes_query = tep_db_query(sprintf(<<<'EOSQL'
SELECT pa.options_id, pa.options_values_id, pa.options_values_price, pa.price_prefix
  FROM products_attributes pa
  WHERE pa.products_id = %s
EOSQL
        , (int)$_GET['products_id']));

      while ( $products_attributes_values = $products_attributes_query->fetch_assoc() ) {
        $option_price = $products_attributes_values['options_values_price'];
        if ($products_attributes_values['price_prefix'] == "-") $option_price= -1*$option_price;
          $attributes_price[$products_attributes_values['options_id']][$products_attributes_values['options_values_id']] = $option_price;
      }

      //OK! time to generate the html table
      //$html_ev_out will be displayed at the end of the script if $rowscounter > 0
      $html_ev_out = '<table class="table table-striped table-bordered table-condensed">
                        <thead>
                          <tr>';

      // build heading line with option names
      while ( $products_options_name = $products_options_name_query->fetch_assoc() ) {
        $html_ev_out .= '     <th class="text-center">' . $products_options_name['products_options_name'] . '</th>';
      }
      $html_ev_out .= '       <th class="text-center">'. PI_QTPRO_TABLE_PRICE .'</th>';
      $html_ev_out .= '       <th class="text-center">'. PI_QTPRO_TABLE_STOCK .'</th>';
      $html_ev_out .= '     </tr>';
      $html_ev_out .= '   </thead>';


      // now create the rows! Each row will display the quantity for one combination of attributes.
      while ( $products_stock_values = $products_stock_query->fetch_assoc() ) {
        if ($products_stock_values['products_stock_quantity'] > 0) {
          //We only want to display rows for combinations we have on stock...
          //For example the quantity can be 0 or even negative if oversold.
          $attributes = explode(',', $products_stock_values['products_stock_attributes']);
          $html_ev_out .= '<tr>';


          $total_price = $products_facts['products_price'];
          foreach ($attributes as $attribute) {
            $attr = explode('-', $attribute);
            $html_ev_out .= '<td class="text-center">' . tep_values_name($attr[1]) . '</td>';
            if ( isset ($attributes_price[$attr[0]][$attr[1]]) ) {
              $total_price  +=  $attributes_price[$attr[0]][$attr[1]];
            }
          }
          $total_price = $GLOBALS['currencies']->display_price($total_price, tep_get_tax_rate($products_facts['products_tax_class_id']));

          $html_ev_out .= '<td class="text-center">' . $total_price . '</td>';
          $html_ev_out .= '<td class="text-center">' . $products_stock_values['products_stock_quantity'] . '</td>';
        }
      }

      $html_ev_out .= '</tr></table>'; //Table is finished!

      if (mysqli_num_rows($products_stock_query) > 0) {
        $tpl_data = ['group' => $this->group, 'file' => __FILE__];
        include 'includes/modules/block_template.php';
      }
    }

    protected function get_parameters() {
      return [
        'PI_QTPRO_TABLE_STATUS' => [
          'title' => 'Enable QT Pro Stock Table Module',
          'value' => 'True',
          'desc' => 'Should this module be shown on the product info page?',
          'set_func' => "tep_cfg_select_option(['True', 'False'], ",
        ],
        'PI_QTPRO_TABLE_GROUP' => [
          'title' => 'Module Display',
          'value' => 'C',
          'desc' => 'Where should this module display on the product info page?',
          'set_func' => "tep_cfg_select_option(['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I'], ",
        ],
        'PI_QTPRO_TABLE_CONTENT_WIDTH' => [
          'title' => 'Content Width',
          'value' => '12',
          'desc' => 'What width container should the content be shown in?',
          'set_func' => "tep_cfg_select_option(['12', '11', '10', '9', '8', '7', '6', '5', '4', '3', '2', '1'], ",
        ],
        'PI_QTPRO_TABLE_SORT_ORDER' => [
          'title' => 'Sort Order',
          'value' => '80',
          'desc' => 'Sort order of display. Lowest is displayed first.',
        ],
      ];
    }

  }

  /*Copied from admin functions*/
  if(!function_exists('tep_values_name')) {
    function tep_values_name($values_id) {

      $values = tep_db_query("SELECT products_options_values_name
                              FROM products_options_values
                              WHERE products_options_values_id = '" . (int)$values_id . "'
                              AND language_id = '" . (int)$GLOBALS['languages_id'] . "'");
      $values_values = tep_db_fetch_array($values);

      return $values_values['products_options_values_name'];
    }
  }