<?php
/*
  QTPro 7.3.1 Phoenix
  by @raiwa
  info@oscaddons.com
  www.oscaddons.com

  Copyright (c) 2021 Rainer Schmied
  All rights reserved.

  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

  3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

class hook_shop_siteWide_qtPro {

  public $version = '7.3.1';

  function listen_cartOrderProductColumns($parameters) {

    $attributes_sql = &$parameters['attributes_sql'];
    $attributes_sql = <<<'EOSQL'
SELECT
    popt.products_options_name AS `option`,
    popt.products_options_track_stock AS `track_stock`,
    poval.products_options_values_name AS value,
    pa.options_id AS option_id,
    pa.options_values_id AS value_id,
    pa.price_prefix AS prefix,
    pa.options_values_price AS price
  FROM products_options popt
    INNER JOIN products_attributes pa ON pa.options_id = popt.products_options_id
    INNER JOIN products_options_values poval ON pa.options_values_id = poval.products_options_values_id AND popt.language_id = poval.language_id
  WHERE pa.products_id = %d AND pa.options_id = %d AND pa.options_values_id = %d AND popt.language_id = %d
EOSQL;
  }

  function listen_databaseOrderProductColumns($parameters) {

    $order_products = &$parameters['column_keys'];
    $order_products['products_stock_attributes'] = 'products_stock_attributes';
  }

  function listen_injectRedirects() {
    global $order;

    if ($GLOBALS['PHP_SELF'] == 'checkout_payment.php') {

      if ( (STOCK_CHECK == 'true') && (STOCK_ALLOW_CHECKOUT != 'true') ) {
        $products = $_SESSION['cart']->get_products();
        $any_out_of_stock = 0;
        for ($i=0, $n=sizeof($products); $i<$n; $i++) {
          if (isset($products[$i]['attributes']) && is_array($products[$i]['attributes'])) {
            $stock_check = $this->check_stock_qtpro($products[$i]['id'], $products[$i]['quantity'], $products[$i]['attributes']);
          } else {
            $stock_check = $this->check_stock_qtpro($products[$i]['id'], $products[$i]['quantity']);
          }

          if ($stock_check) $any_out_of_stock = 1;
        }
        if ($any_out_of_stock == 1) {
          tep_redirect(tep_href_link('shopping_cart.php'));
        }
      }
    } elseif ($GLOBALS['PHP_SELF'] == 'checkout_confirmation.php') {

      $any_out_of_stock = false;
      if (STOCK_CHECK == 'true') {
        $check_stock = [];
        for ($i = 0, $n = sizeof($order->products); $i<$n; $i++) {
          if (isset($order->products[$i]['attributes']) && is_array($order->products[$i]['attributes'])) {
            $attributes = [];
            foreach ($order->products[$i]['attributes'] as $attribute) {
              $attributes[$attribute['option_id']]=$attribute['value_id'];
            }
            $check_stock[$i] = $this->check_stock_qtpro($order->products[$i]['id'], $order->products[$i]['qty'], $attributes);
          } else {
            $check_stock[$i] = $this->check_stock_qtpro($order->products[$i]['id'], $order->products[$i]['qty']);
          }
          if ($check_stock[$i]) {
            $any_out_of_stock = true;
          }
        }    // Out of Stock
        if ( (STOCK_ALLOW_CHECKOUT != 'true') && ($any_out_of_stock == true) ) {
          tep_redirect(tep_href_link('shopping_cart.php'));
        }
      }
    }

  }
  
  function listen_insertOrder($parameters) {
    global $order;
          
    $orders = &$parameters['sql_data']['orders_products'];   
    
    $any_out_of_stock = false;
    if (STOCK_CHECK == 'true') {
      $check_stock = [];
      for ($i = 0, $n = sizeof($order->products); $i<$n; $i++) {
        if (isset($order->products[$i]['attributes']) && is_array($order->products[$i]['attributes'])) {
          $attributes = [];
          foreach ($order->products[$i]['attributes'] as $attribute) {
            $attributes[$attribute['option_id']]=$attribute['value_id'];
          }
          $check_stock[$i] = $this->check_stock_qtpro($order->products[$i]['id'], $order->products[$i]['qty'], $attributes);
        } else {
          $check_stock[$i] = $this->check_stock_qtpro($order->products[$i]['id'], $order->products[$i]['qty']);
        }
        if ($check_stock[$i]) {
          $any_out_of_stock = true;
        }
      }    // Out of Stock
      if ( (STOCK_ALLOW_CHECKOUT != 'true') && ($any_out_of_stock == true) ) {
        tep_redirect(tep_href_link('shopping_cart.php'));
      }
      $products_stock_attributes = null;

      if (STOCK_LIMITED == 'true') {
        for ($i = 0, $n = sizeof($order->products); $i<$n; $i++) {
          $products_attributes = $order->products[$i]['attributes'] ?? '';
          if (DOWNLOAD_ENABLED == 'true') {
            $stock_query_raw = sprintf(<<<'EOSQL'
SELECT products_quantity, pad.products_attributes_filename 
  FROM products p 
  LEFT JOIN products_attributes pa 
   ON p.products_id = pa.products_id 
  LEFT JOIN products_attributes_download pad 
   ON pa.products_attributes_id = pad.products_attributes_id 
  WHERE p.products_id = %s 
EOSQL
            , tep_get_prid($order->products[$i]['id']));

    // Will work with only one option for downloadable products
    // otherwise, we have to build the query dynamically with a loop
            if (is_array($products_attributes)) {
              $stock_query_raw .= sprintf(<<<'EOSQL'
    AND pa.options_id = %s 
    AND pa.options_values_id = %s
EOSQL
          , (int)$products_attributes[0]['option_id'], (int)$products_attributes[0]['value_id']);
    
            }
            $stock_query = tep_db_query($stock_query_raw);
          } else {
            $stock_query = tep_db_query(sprintf(<<<'EOSQL'
SELECT products_quantity 
  FROM products 
  WHERE products_id = %s
EOSQL
          , tep_get_prid($order->products[$i]['id'])));

          }
          if (mysqli_num_rows($stock_query) > 0) {
            $stock_values = $stock_query->fetch_assoc();
            $actual_stock_bought = $order->products[$i]['qty'];
            $download_selected = false;
            if ((DOWNLOAD_ENABLED == 'true') && isset($stock_values['products_attributes_filename']) && tep_not_null($stock_values['products_attributes_filename'])) {
              $download_selected = true;
              $products_stock_attributes = '$$DOWNLOAD$$';
            }
          }
// If not downloadable and attributes present, adjust attribute stock
          if ( !$download_selected && is_array($products_attributes) ) {
            $all_nonstocked = true;
            $products_stock_attributes_array = [];
            foreach ($products_attributes as $attribute) {
              if (isset($attribute['track_stock']) && $attribute['track_stock'] == 1) {
                $products_stock_attributes_array[] = $attribute['option_id'] . "-" . $attribute['value_id'];
                $all_nonstocked = false;
              }
            }
// create $products_stock_attributes_array if PayPAl standard IPN from orders products table            
            if (isset($product['products_stock_attributes'])) {
              echo '<br><br>$product' . print_r(explode(',', $product['products_stock_attributes'])) . '<br>';
              $products_stock_attributes_array = explode(',', $product['products_stock_attributes']);
              $all_nonstocked = false;
            }
            
            if ($all_nonstocked) {
              $actual_stock_bought = $order->products[$i]['qty'];
            } else {
              asort($products_stock_attributes_array, SORT_NUMERIC);
              $products_stock_attributes = implode(",", $products_stock_attributes_array);
              $attributes_stock_query = tep_db_query(sprintf(<<<'EOSQL'
SELECT products_stock_quantity 
  FROM products_stock 
  WHERE products_stock_attributes = '%s' 
    AND products_id = %s
EOSQL
              , $products_stock_attributes, tep_get_prid($order->products[$i]['id'])));
            }
          }
          $orders[$i]['products_stock_attributes'] = $products_stock_attributes;
        }
      }
    }
  }

  function listen_afterStart() {
    global $order;

    foreach ($order->products as $i => $product) {    
      $products_stock_attributes = null;

      if (STOCK_LIMITED == 'true') {
        $products_attributes = $order->products[$i]['attributes'] ?? '';
        if (DOWNLOAD_ENABLED == 'true') {
          $stock_query_raw = sprintf(<<<'EOSQL'
SELECT products_quantity, pad.products_attributes_filename 
  FROM products p 
  LEFT JOIN products_attributes pa 
   ON p.products_id = pa.products_id 
  LEFT JOIN products_attributes_download pad 
   ON pa.products_attributes_id = pad.products_attributes_id 
  WHERE p.products_id = %s 
EOSQL
          , tep_get_prid($order->products[$i]['id']));

  // Will work with only one option for downloadable products
  // otherwise, we have to build the query dynamically with a loop
          if (is_array($products_attributes)) {
            $stock_query_raw .= sprintf(<<<'EOSQL'
    AND pa.options_id = %s 
    AND pa.options_values_id = %s
EOSQL
        , (int)$products_attributes[0]['option_id'], (int)$products_attributes[0]['value_id']);
    
          }
          $stock_query = tep_db_query($stock_query_raw);
        } else {
          $stock_query = tep_db_query(sprintf(<<<'EOSQL'
SELECT products_quantity 
  FROM products 
  WHERE products_id = %s
EOSQL
        , tep_get_prid($order->products[$i]['id'])));

        }
        if (mysqli_num_rows($stock_query) > 0) {
          $stock_values = $stock_query->fetch_assoc();
          $actual_stock_bought = $order->products[$i]['qty'];
          $download_selected = false;
          if ((DOWNLOAD_ENABLED == 'true') && isset($stock_values['products_attributes_filename']) && tep_not_null($stock_values['products_attributes_filename'])) {
            $download_selected = true;
            $products_stock_attributes = '$$DOWNLOAD$$';
          }
          
// If not downloadable and attributes present, adjust attribute stock
          if ( !$download_selected && is_array($products_attributes) ) {
            $all_nonstocked = true;
            $products_stock_attributes_array = [];
            foreach ($products_attributes as $attribute) {
              if (isset($attribute['track_stock']) && $attribute['track_stock'] == 1) {
                $products_stock_attributes_array[] = $attribute['option_id'] . "-" . $attribute['value_id'];
                $all_nonstocked = false;
              }
            }
// create $products_stock_attributes_array if PayPAl standard IPN from orders products table            
            if (isset($product['products_stock_attributes'])) {
              $products_stock_attributes_array = explode(',', $product['products_stock_attributes']);
              $all_nonstocked = false;
            }
            
            if ($all_nonstocked) {
              $actual_stock_bought = $order->products[$i]['qty'];
            } else {
              asort($products_stock_attributes_array, SORT_NUMERIC);
              $products_stock_attributes = implode(",", $products_stock_attributes_array);
              $attributes_stock_query = tep_db_query(sprintf(<<<'EOSQL'
SELECT products_stock_quantity 
  FROM products_stock 
  WHERE products_stock_attributes = '%s' 
    AND products_id = %s
EOSQL
              , $products_stock_attributes, tep_get_prid($order->products[$i]['id'])));

              if (mysqli_num_rows($attributes_stock_query) > 0) {
                $attributes_stock_values = tep_db_fetch_array($attributes_stock_query);
                $attributes_stock_left = $attributes_stock_values['products_stock_quantity'] - $order->products[$i]['qty'];
                tep_db_query(sprintf(<<<'EOSQL'
UPDATE products_stock 
  SET products_stock_quantity = %s 
  WHERE products_stock_attributes = '%s' 
    AND products_id = %s
EOSQL
                , $attributes_stock_left, $products_stock_attributes, tep_get_prid($order->products[$i]['id'])));

                $actual_stock_bought = ($attributes_stock_left < 1) ? $attributes_stock_values['products_stock_quantity'] : $order->products[$i]['qty'];
              } else {
                $attributes_stock_left = 0 - $order->products[$i]['qty'];
                tep_db_query(sprintf(<<<'EOSQL'
INSERT INTO products_stock (products_id, products_stock_attributes, products_stock_quantity) 
  VALUES (%s, '%s', %s) 
EOSQL
                , tep_get_prid($order->products[$i]['id']), $products_stock_attributes, $attributes_stock_left));

                $actual_stock_bought = 0;
              }
            }
          }
        }
      }
    }
  
  }

  ////
  // Check if the required stock is available
  // If insufficent stock is available return $out_of_stock = true
  public function check_stock_qtpro($products_id, $products_quantity, $attributes = []) {
    $stock_left = $this->get_products_stock_qtpro($products_id, $attributes) - $products_quantity;
    $out_of_stock = '';
    if ($stock_left < 0) {
      $out_of_stock = '<span class="text-danger"><b>' . STOCK_MARK_PRODUCT_OUT_OF_STOCK . '</b></span>';
    }
    return $out_of_stock;
  }

  ////
  // Return a product's stock
  // TABLES: products. products_stock
  private function get_products_stock_qtpro($products_id, $attributes = []) {

    $products_id = tep_get_prid($products_id);
    $all_nonstocked = true;
    if (sizeof($attributes) > 0) {
      $attr_list = '';
      $options_list = implode(", ", array_keys($attributes));
      $track_stock_query = tep_db_query(sprintf(<<<'EOSQL'
SELECT products_options_id, products_options_track_stock
  FROM products_options
  WHERE products_options_id IN (%s)
    AND language_id= %s
  ORDER BY products_options_id
EOSQL
        , $options_list, (int)$GLOBALS['languages_id']));

      while ( $track_stock_array = $track_stock_query->fetch_assoc() ) {
        if ( $track_stock_array['products_options_track_stock'] == '1') {
          $attr_list .= $track_stock_array['products_options_id'] . '-' . $attributes[$track_stock_array['products_options_id']] . ',';
          $all_nonstocked = false;
        }
      }
      $attr_list = substr($attr_list, 0, strlen($attr_list)-1);
    }

    if ( (sizeof($attributes) == 0) || ($all_nonstocked) ) {
      $stock_query = tep_db_query(sprintf(<<<'EOSQL'
SELECT products_quantity as quantity
  FROM products
  WHERE products_id = %s
EOSQL
        , (int)$products_id));
} else {
      $stock_query = tep_db_query(sprintf(<<<'EOSQL'
SELECT products_stock_quantity AS quantity
  FROM products_stock
  WHERE products_id = %s
    AND products_stock_attributes = '%s'
EOSQL
        , (int)$products_id, $attr_list));
    }
    if ( mysqli_num_rows($stock_query) > 0 ) {
      $stock = $stock_query->fetch_assoc();
      $quantity = $stock['quantity'];
    } else {
      $quantity = 0;
    }
    return $quantity;
  }

}
